@@ -3,59 +3,71 @@
 //!
 //! ## Example usage
 //! ```
-//! use roffman::{IntoRoffNode, Roff, Roffable, RoffNode};
+//! use roffman::{Roff, RoffNode, Roffable, SectionNumber, SynopsisOpt};
 //!
-//! let roff = Roff::new("roffman", 7)
-//!     .date("August 2021")
-//!     .section(
-//!        "BASIC USAGE",
-//!        vec![
-//!            RoffNode::paragraph(vec![
-//!                "This is how you create a basic paragraph using roffman.",
-//!            ]),
-//!            RoffNode::indented_paragraph(
-//!                vec![
-//!                    "This line should be slightly indented to the ".roff(),
-//!                    "right.".roff().bold(),
-//!                ],
-//!                Some(4),
-//!            ),
-//!            RoffNode::paragraph(vec![
-//!                "And some example ".roff(),
-//!                "code".roff().italic(),
-//!                ":".roff(),
-//!            ]),
-//!            RoffNode::example(vec![
-//!                r#"
+//! let roff = Roff::new("roffman", SectionNumber::Miscellaneous)
+//! .date("August 2021")
+//! .section(
+//!    "BASIC USAGE",
+//!    [
+//!        RoffNode::paragraph([
+//!            "This is how you create a basic paragraph using roffman.",
+//!        ]),
+//!        RoffNode::indented_paragraph(
+//!            [
+//!                "This line should be slightly indented to the ".roff(),
+//!                "right.".roff().bold(),
+//!            ],
+//!            Some(4),
+//!        ),
+//!        RoffNode::synopsis(
+//!                 "roffman-command",
+//!                 [
+//!                 "This is the description of this command. It will be displayed right next to".roff(),
+//! " it".roff().italic()
+//! ]                     ,
+//!                 [
+//!                 SynopsisOpt::new("--opt").description(["some simple opt"]),
+//!                 SynopsisOpt::new("--opt-with-arg").argument("ARG").description(["opt with an argument"]),
+//!                 SynopsisOpt::new("--bold")
+//!        ]),
+//!        RoffNode::paragraph(["Example:".roff().bold()]),
+//!        RoffNode::example([
+//!             r#"
 //! impl Roffable for u8 {
 //!     fn roff(&self) -> RoffText {
 //!         self.to_string().roff()
 //!     }
 //! }"#,
-//!             ]),
-//!         ],
-//!     );
+//!         ]),
+//!        RoffNode::url("GitHub", "https://github.com/vv9k/roffman"),
+//!        RoffNode::text("\nvv9k"),
+//!        RoffNode::trademark_sign(),
+//!     ],
+//! );
 //!
 //! let rendered = roff.to_string().unwrap();
 //!
-//! let output = r#"
-//! .TH "roffman" "7" "August 2021"
-//! .
-//!
+//! let output = r#".TH roffman 7 "August 2021"
 //! .SH "BASIC USAGE"
-//!
 //! .P
 //! This is how you create a basic paragraph using roffman\.
-//! .
-//!
 //! .IP "" 4
 //! This line should be slightly indented to the \fBright\.\fR
-//! .
+//! .SY roffman\-command
+//! This is the description of this command\. It will be displayed right next to \fIit\fR
 //!
+//! .OP \-\-opt
+//! some simple opt
+//!
+//! .OP \-\-opt\-with\-arg ARG
+//! opt with an argument
+//!
+//! .OP \-\-bold
+//!
+//! .YS
 //! .P
-//! And some example \fIcode\fR:
-//! .
-//!
+//! \fBExample:\fR
 //! .EX
 //!
 //! impl Roffable for u8 {
@@ -63,30 +75,45 @@
 //!         self\.to_string()\.roff()
 //!     }
 //! }
-//! .EE"#;
+//! .EE
+//! .UR https://github\.com/vv9k/roffman
+//! GitHub
+//! .UE
+//!
+//! vv9k\*(Tm
+//! "#;
 //!
 //! assert_eq!(rendered.trim(), output.trim());
 //! ```
 //!
-//!
 //! which will look something like this:
 //! ```text
-//! roffman(7)            Miscellaneous Information Manual           roffman(7)
+//! roffman(7)                                         Miscellaneous Information Manual                                  roffman(7)
 //!
 //! BASIC USAGE
 //!        This is how you create a basic paragraph using roffman.
 //!
 //!            This line should be slightly indented to the right.
 //!
-//!        And some example code:
+//!        roffman-command This is the description of this command. It will be displayed right next to it
+//!
+//!                        [--opt] some simple opt
+//!
+//!                        [--opt-with-arg ARG] opt with an argument
+//!
+//!                        [--bold]
+//!
+//!        Example:
 //!
 //!        impl Roffable for u8 {
 //!            fn roff(&self) -> RoffText {
 //!                self.to_string().roff()
 //!            }
 //!        }
+//!        GitHub ⟨https://github.com/vv9k/roffman⟩
 //!
-//!                                                                  roffman(7)
+//!        vv9k™
+//!                                                               August 2021                                             roffman(7)
 //! ```
 
 use std::error::Error;
@@ -94,7 +121,6 @@ use std::fmt;
 use std::fmt::Formatter;
 use std::io::{self, Write};
 
-const COMMA: &[u8] = b"\n.\n";
 const SPACE: &[u8] = b" ";
 const QUOTE: &[u8] = b"\"";
 const ENDL: &[u8] = b"\n";
@@ -111,6 +137,17 @@ const NESTED_START: &[u8] = b".RS";
 const NESTED_END: &[u8] = b".RE";
 const EXAMPLE_START: &[u8] = b".EX";
 const EXAMPLE_END: &[u8] = b".EE";
+const SYNOPSIS_START: &[u8] = b".SY";
+const SYNOPSIS_END: &[u8] = b".YS";
+const SYNOPSIS_OPT: &[u8] = b".OP";
+const URL_START: &[u8] = b".UR";
+const URL_END: &[u8] = b".UE";
+const MAIL_START: &[u8] = b".MT";
+const MAIL_END: &[u8] = b".ME";
+const LEFT_QUOTE: &[u8] = b"\\*(lq";
+const RIGHT_QUOTE: &[u8] = b"\\*(rq";
+const REGISTERED_SIGN: &[u8] = b"\\*R";
+const TRADEMARK_SIGN: &[u8] = b"\\*(Tm";
 
 #[derive(Debug)]
 /// An error type returned by the functions used in this crate.
@@ -158,19 +195,27 @@ fn write_quoted(roff: &RoffText, writer: &mut impl Write) -> Result<(), RoffErro
     Ok(())
 }
 
+fn write_quoted_if_whitespace(roff: &RoffText, writer: &mut impl Write) -> Result<(), RoffError> {
+    if roff.content.as_bytes().iter().any(u8::is_ascii_whitespace) {
+        write_quoted(roff, writer)
+    } else {
+        roff.render(writer)
+    }
+}
+
 #[derive(Clone, Debug)]
 /// Represents a ROFF document that can be rendered and displayed
 /// with tools like [`man`](https://man7.org/linux/man-pages/man1/man.1.html).
 pub struct Roff {
     title: RoffText,
     date: Option<RoffText>,
-    section: u8,
+    section: SectionNumber,
     sections: Vec<Section>,
 }
 
 impl Roff {
     /// Create a new `Roff` with a `title` and a `section`.
-    pub fn new(title: impl Roffable, section: u8) -> Self {
+    pub fn new(title: impl Roffable, section: SectionNumber) -> Self {
         Self {
             title: title.roff(),
             date: None,
@@ -216,18 +261,18 @@ impl Roff {
 
     fn write_title(&self, writer: &mut impl Write) -> Result<(), RoffError> {
         writer.write_all(SPACE)?;
-        write_quoted(&self.title, writer)
+        write_quoted_if_whitespace(&self.title, writer)
     }
 
     fn write_section(&self, writer: &mut impl Write) -> Result<(), RoffError> {
         writer.write_all(SPACE)?;
-        write_quoted(&self.section.roff(), writer)
+        write_quoted_if_whitespace(&self.section.roff(), writer)
     }
 
     fn write_date(&self, writer: &mut impl Write) -> Result<(), RoffError> {
         if let Some(date) = &self.date {
             writer.write_all(SPACE)?;
-            write_quoted(date, writer)?;
+            write_quoted_if_whitespace(date, writer)?;
         }
         Ok(())
     }
@@ -237,7 +282,7 @@ impl Roff {
         self.write_title(writer)?;
         self.write_section(writer)?;
         self.write_date(writer)?;
-        writer.write_all(COMMA)?;
+        writer.write_all(ENDL)?;
         Ok(())
     }
 
@@ -253,6 +298,53 @@ impl Roff {
     }
 }
 
+#[derive(Copy, Clone, Debug, PartialEq)]
+/// Defines the section to which the given ROFF belongs.
+pub enum SectionNumber {
+    ///Commands that can be executed by the user from within a shell.
+    UserCommands,
+    /// Functions which wrap operations performed by the kernel.
+    SystemCalls,
+    /// All library functions excluding the system call wrappers (Most of the libc functions).
+    LibraryCalls,
+    /// Files found in `/dev` which allow to access to devices through the kernel.
+    Devices,
+    /// Describes various human-readable file formats and configuration files.
+    FileFormatsAndConfigurationFiles,
+    /// Games and funny little programs available on the system.
+    Games,
+    /// Overviews or descriptions of various topics, conventions, and protocols, character set
+    /// standards, the standard filesystem layout, and miscellaneous other things.
+    Miscellaneous,
+    /// Commands like `mount(8)`, many of which only root can execute.
+    SystemManagementCommands,
+    /// A custom section number.
+    Custom(u8),
+}
+
+impl From<SectionNumber> for u8 {
+    fn from(s: SectionNumber) -> Self {
+        use SectionNumber::*;
+        match s {
+            UserCommands => 1,
+            SystemCalls => 2,
+            LibraryCalls => 3,
+            Devices => 4,
+            FileFormatsAndConfigurationFiles => 5,
+            Games => 6,
+            Miscellaneous => 7,
+            SystemManagementCommands => 8,
+            Custom(n) => n,
+        }
+    }
+}
+
+impl Roffable for SectionNumber {
+    fn roff(&self) -> RoffText {
+        u8::from(*self).roff()
+    }
+}
+
 #[derive(Clone, Debug)]
 /// A single section of the ROFF document.
 pub struct Section {
@@ -285,51 +377,50 @@ impl Section {
     }
 
     fn render<W: Write>(&self, writer: &mut W) -> Result<(), RoffError> {
-        writer.write_all(ENDL)?;
         writer.write_all(SECTION_HEADER)?;
         writer.write_all(SPACE)?;
-        write_quoted(&self.title, writer)?;
+        write_quoted_if_whitespace(&self.title, writer)?;
         writer.write_all(ENDL)?;
         if let Some(subtitle) = &self.subtitle {
             writer.write_all(SUB_HEADER)?;
             writer.write_all(SPACE)?;
-            writer.write_all(QUOTE)?;
-            subtitle.render(writer)?;
-            writer.write_all(QUOTE)?;
+            write_quoted_if_whitespace(subtitle, writer)?;
             writer.write_all(ENDL)?;
         }
 
+        let mut was_text = false;
         for node in &self.nodes {
-            node.render(writer, false)?;
+            node.render(writer, false, was_text)?;
+            was_text = node.is_text();
         }
 
         Ok(())
     }
 }
 
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, PartialEq)]
 /// Style that can be applied to [`RoffText`](RoffText)
-pub enum Style {
+pub enum FontStyle {
     Bold,
     Italic,
-    Normal,
+    Roman,
 }
 
-impl Default for Style {
+impl Default for FontStyle {
     fn default() -> Self {
-        Style::Normal
+        FontStyle::Roman
     }
 }
 
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Debug, Default, PartialEq)]
 /// Wrapper type for styled text in ROFF.
 pub struct RoffText {
     content: String,
-    style: Style,
+    style: FontStyle,
 }
 
 impl RoffText {
-    pub fn new<C: AsRef<str>>(content: C, style: Option<Style>) -> Self {
+    pub fn new<C: AsRef<str>>(content: C, style: Option<FontStyle>) -> Self {
         Self {
             content: escape(content),
             style: style.unwrap_or_default(),
@@ -338,27 +429,27 @@ impl RoffText {
 
     /// Set the style of this text to bold.
     pub fn bold(mut self) -> Self {
-        self.style = Style::Bold;
+        self.style = FontStyle::Bold;
         self
     }
 
     /// Set the style of this text to italic.
     pub fn italic(mut self) -> Self {
-        self.style = Style::Italic;
+        self.style = FontStyle::Italic;
         self
     }
 
     fn render<W: Write>(&self, writer: &mut W) -> Result<(), RoffError> {
         let styled = match self.style {
-            Style::Bold => {
+            FontStyle::Bold => {
                 writer.write_all(BOLD)?;
                 true
             }
-            Style::Italic => {
+            FontStyle::Italic => {
                 writer.write_all(ITALIC)?;
                 true
             }
-            Style::Normal => false,
+            FontStyle::Roman => false,
         };
 
         writer.write_all(self.content.as_bytes())?;
@@ -380,6 +471,7 @@ impl RoffNode {
         self.0
     }
 
+    /// Creates a simple text node.
     pub fn text(content: impl Roffable) -> Self {
         Self(RoffNodeInner::Text(content.roff()))
     }
@@ -429,7 +521,7 @@ impl RoffNode {
         })
     }
 
-    /// Creates a new example node.
+    /// Creates a new example node. An example block usually has the font set to monospaced.
     pub fn example<I, R>(content: I) -> Self
     where
         I: IntoIterator<Item = R>,
@@ -439,6 +531,93 @@ impl RoffNode {
             content.into_iter().map(|item| item.roff()).collect(),
         ))
     }
+
+    /// Creates a new synopsis node explaining the given `command` with `description` and `opts`.
+    pub fn synopsis<I, R, O>(command: impl Roffable, description: I, opts: O) -> Self
+    where
+        I: IntoIterator<Item = R>,
+        R: Roffable,
+        O: IntoIterator<Item = SynopsisOpt>,
+    {
+        Self(RoffNodeInner::Synopsis {
+            command: command.roff(),
+            text: description.into_iter().map(|item| item.roff()).collect(),
+            opts: opts.into_iter().collect(),
+        })
+    }
+
+    /// Creates a new URL node that will take the form of `[name](address)` where `name` is the
+    /// visible part of the URL and address is where it points to.
+    pub fn url(name: impl Roffable, address: impl Roffable) -> Self {
+        Self(RoffNodeInner::Url {
+            name: name.roff(),
+            address: address.roff(),
+        })
+    }
+
+    /// Creates a new email node that will where `address` is the email address and `name` is the
+    /// visible URL text. `address` may not be visible if the man page is being viewed as HTML.
+    pub fn email(name: impl Roffable, address: impl Roffable) -> Self {
+        Self(RoffNodeInner::Email {
+            name: name.roff(),
+            address: address.roff(),
+        })
+    }
+
+    /// Returns a node that will be rendered as a registered sign `®`.
+    pub fn registered_sign() -> Self {
+        Self(RoffNodeInner::RegisteredSign)
+    }
+
+    /// Returns a node that will be rendered as a left quote `“`.
+    pub fn left_quote() -> Self {
+        Self(RoffNodeInner::LeftQuote)
+    }
+
+    /// Returns a node that will be rendered as a right quote `”`.
+    pub fn right_quote() -> Self {
+        Self(RoffNodeInner::RightQuote)
+    }
+
+    /// Returns a node that will be rendered as a trademark sign `™`.
+    pub fn trademark_sign() -> Self {
+        Self(RoffNodeInner::TrademarkSign)
+    }
+}
+
+#[derive(Clone, Debug)]
+/// An option used by the [`RoffNode::synopsis`](RoffNode::synopsis) block.
+pub struct SynopsisOpt {
+    name: RoffText,
+    argument: Option<RoffText>,
+    description: Option<Vec<RoffText>>,
+}
+
+impl SynopsisOpt {
+    /// Creates a new option used in a synopsis block.
+    pub fn new<R: Roffable>(name: R) -> Self {
+        Self {
+            name: name.roff(),
+            argument: None,
+            description: None,
+        }
+    }
+
+    /// Set the name of the argument that this option takes.
+    pub fn argument<R: Roffable>(mut self, argument: R) -> Self {
+        self.argument = Some(argument.roff());
+        self
+    }
+
+    /// Set the description for this command synopsis.
+    pub fn description<I, R>(mut self, description: I) -> Self
+    where
+        I: IntoIterator<Item = R>,
+        R: Roffable,
+    {
+        self.description = Some(description.into_iter().map(|item| item.roff()).collect());
+        self
+    }
 }
 
 #[derive(Clone, Debug)]
@@ -461,15 +640,41 @@ enum RoffNodeInner {
     },
     /// An example block where text is monospaced.
     Example(Vec<RoffText>),
+    Synopsis {
+        command: RoffText,
+        text: Vec<RoffText>,
+        opts: Vec<SynopsisOpt>,
+    },
+    Url {
+        name: RoffText,
+        address: RoffText,
+    },
+    Email {
+        name: RoffText,
+        address: RoffText,
+    },
+    RegisteredSign,
+    LeftQuote,
+    RightQuote,
+    TrademarkSign,
 }
 
 impl RoffNodeInner {
     /// Returns `true` if the node is the [`RoffNode::Text`](RoffNode::Text) variant.
-    pub fn is_text(&self) -> bool {
+    fn is_text(&self) -> bool {
         matches!(self, &RoffNodeInner::Text(_))
     }
 
-    fn render<W: Write>(&self, writer: &mut W, nested: bool) -> Result<(), RoffError> {
+    fn is_nestable(&self) -> bool {
+        !self.is_text()
+    }
+
+    fn render<W: Write>(
+        &self,
+        writer: &mut W,
+        nested: bool,
+        was_text: bool,
+    ) -> Result<(), RoffError> {
         if nested {
             writer.write_all(ENDL)?;
             writer.write_all(NESTED_START)?;
@@ -477,15 +682,15 @@ impl RoffNodeInner {
         match self {
             RoffNodeInner::Text(text) => {
                 let styled = match text.style {
-                    Style::Bold => {
+                    FontStyle::Bold => {
                         writer.write_all(BOLD)?;
                         true
                     }
-                    Style::Italic => {
+                    FontStyle::Italic => {
                         writer.write_all(ITALIC)?;
                         true
                     }
-                    Style::Normal => false,
+                    FontStyle::Roman => false,
                 };
 
                 writer.write_all(text.content.as_bytes())?;
@@ -494,46 +699,60 @@ impl RoffNodeInner {
                 }
             }
             RoffNodeInner::Paragraph(content) => {
-                writer.write_all(ENDL)?;
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
                 writer.write_all(PARAGRAPH)?;
                 writer.write_all(ENDL)?;
+                let mut was_text_node = false;
                 for node in content {
-                    node.render(writer, !node.is_text())?;
+                    node.render(writer, node.is_nestable(), was_text_node)?;
+                    was_text_node = node.is_text();
                 }
-                writer.write_all(COMMA)?;
+                writer.write_all(ENDL)?;
             }
             RoffNodeInner::IndentedParagraph {
                 content,
                 indentation,
             } => {
-                writer.write_all(ENDL)?;
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
                 writer.write_all(INDENTED_PARAGRAPH)?;
                 if let Some(indentation) = indentation {
                     writer.write_all(format!(" \"\" {}", indentation).as_bytes())?;
                 }
                 writer.write_all(ENDL)?;
+                let mut was_text_node = false;
                 for node in content {
-                    node.render(writer, !node.is_text())?;
+                    node.render(writer, node.is_nestable(), was_text_node)?;
+                    was_text_node = node.is_text();
                 }
-                writer.write_all(COMMA)?;
+                writer.write_all(ENDL)?;
             }
             RoffNodeInner::TaggedParagraph {
                 content,
                 title: tag,
             } => {
-                writer.write_all(ENDL)?;
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
                 writer.write_all(TAGGED_PARAGRAPH)?;
                 writer.write_all(ENDL)?;
                 tag.render(writer)?;
                 writer.write_all(ENDL)?;
 
+                let mut was_text_node = false;
                 for node in content {
-                    node.render(writer, !node.is_text())?;
+                    node.render(writer, node.is_nestable(), was_text_node)?;
+                    was_text_node = node.is_text();
                 }
-                writer.write_all(COMMA)?;
+                writer.write_all(ENDL)?;
             }
             RoffNodeInner::Example(content) => {
-                writer.write_all(ENDL)?;
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
                 writer.write_all(EXAMPLE_START)?;
                 writer.write_all(ENDL)?;
                 for node in content {
@@ -541,7 +760,76 @@ impl RoffNodeInner {
                 }
                 writer.write_all(ENDL)?;
                 writer.write_all(EXAMPLE_END)?;
+                writer.write_all(ENDL)?;
             }
+            RoffNodeInner::Synopsis {
+                command,
+                text,
+                opts,
+            } => {
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
+                writer.write_all(SYNOPSIS_START)?;
+                writer.write_all(SPACE)?;
+                write_quoted_if_whitespace(command, writer)?;
+                writer.write_all(ENDL)?;
+                for elem in text {
+                    elem.render(writer)?;
+                }
+                if !text.is_empty() {
+                    writer.write_all(ENDL)?;
+                }
+                for op in opts {
+                    writer.write_all(ENDL)?;
+                    writer.write_all(SYNOPSIS_OPT)?;
+                    writer.write_all(SPACE)?;
+                    write_quoted_if_whitespace(&op.name, writer)?;
+                    if let Some(arg) = &op.argument {
+                        writer.write_all(SPACE)?;
+                        write_quoted_if_whitespace(arg, writer)?;
+                    }
+                    writer.write_all(ENDL)?;
+                    if let Some(description) = &op.description {
+                        for elem in description {
+                            elem.render(writer)?;
+                        }
+                    }
+                    writer.write_all(ENDL)?;
+                }
+                writer.write_all(SYNOPSIS_END)?;
+                writer.write_all(ENDL)?;
+            }
+            RoffNodeInner::Url { address, name } => {
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
+                writer.write_all(URL_START)?;
+                writer.write_all(SPACE)?;
+                address.render(writer)?;
+                writer.write_all(ENDL)?;
+                name.render(writer)?;
+                writer.write_all(ENDL)?;
+                writer.write_all(URL_END)?;
+                writer.write_all(ENDL)?;
+            }
+            RoffNodeInner::Email { address, name } => {
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
+                writer.write_all(MAIL_START)?;
+                writer.write_all(SPACE)?;
+                address.render(writer)?;
+                writer.write_all(ENDL)?;
+                name.render(writer)?;
+                writer.write_all(ENDL)?;
+                writer.write_all(MAIL_END)?;
+                writer.write_all(ENDL)?;
+            }
+            RoffNodeInner::RegisteredSign => writer.write_all(REGISTERED_SIGN)?,
+            RoffNodeInner::LeftQuote => writer.write_all(LEFT_QUOTE)?,
+            RoffNodeInner::RightQuote => writer.write_all(RIGHT_QUOTE)?,
+            RoffNodeInner::TrademarkSign => writer.write_all(TRADEMARK_SIGN)?,
         }
 
         if nested {
@@ -571,21 +859,9 @@ impl IntoRoffNode for RoffNode {
     }
 }
 
-impl IntoRoffNode for RoffText {
+impl<R: Roffable> IntoRoffNode for R {
     fn into_roff(self) -> RoffNode {
-        RoffNode::text(self)
-    }
-}
-
-impl IntoRoffNode for &str {
-    fn into_roff(self) -> RoffNode {
-        self.roff().into_roff()
-    }
-}
-
-impl IntoRoffNode for String {
-    fn into_roff(self) -> RoffNode {
-        self.roff().into_roff()
+        RoffNode::text(self.roff())
     }
 }
 
@@ -601,9 +877,21 @@ impl Roffable for String {
     }
 }
 
+impl Roffable for &String {
+    fn roff(&self) -> RoffText {
+        RoffText::new((*self).clone(), None)
+    }
+}
+
 impl Roffable for &str {
     fn roff(&self) -> RoffText {
-        self.to_string().roff()
+        RoffText::new(self.to_string(), None)
+    }
+}
+
+impl Roffable for &&str {
+    fn roff(&self) -> RoffText {
+        (*self).roff()
     }
 }
 
@@ -615,7 +903,7 @@ impl Roffable for RoffText {
 
 impl Roffable for u8 {
     fn roff(&self) -> RoffText {
-        self.to_string().roff()
+        RoffText::new(self.to_string(), None)
     }
 }
 
@@ -625,7 +913,7 @@ mod tests {
 
     #[test]
     fn it_roffs() {
-        let roff = Roff::new("test", 1)
+        let roff = Roff::new("test", SectionNumber::UserCommands)
             .section(
                 "test section 1",
                 vec![RoffNode::paragraph(vec![
@@ -655,27 +943,17 @@ mod tests {
 
         let rendered = roff.to_string().unwrap();
         assert_eq!(
-            r#".TH "test" "1"
-.
-
+            r#".TH test 1
 .SH "test section 1"
-
 .P
 this is some very \fBspecial\fR text
-.
-
 .SH "test section 2"
-
 .IP "" 4
 \fILorem ipsum\fR dolor sit amet, consectetur adipiscing elit\. Vivamus quis malesuada eros\.
-.
-
 .SH "test section 3"
-
 .TP
 \fBparagraph title\fR
 tagged paragraph with some content
-.
 "#,
             rendered
         )
@@ -683,7 +961,7 @@ tagged paragraph with some content
 
     #[test]
     fn it_nests_roffs() {
-        let roff = Roff::new("test", 1).add_section(
+        let roff = Roff::new("test", SectionNumber::UserCommands).add_section(
             Section::new(
                 "BASE SECTION",
                 vec![
@@ -704,12 +982,9 @@ tagged paragraph with some content
 
         let rendered = roff.to_string().unwrap();
         assert_eq!(
-            r#".TH "test" "1"
-.
-
+            r#".TH test 1
 .SH "BASE SECTION"
 .SS "with some subtitle\.\.\."
-
 .P
 some text in first paragraph\.
 .RS
@@ -718,17 +993,12 @@ some nested paragraph
 .RS
 .P
 some doubly nested paragraph
-.
 
 .RE
-.
 
 .RE
-.
-
 .P
 back two levels left without roffs
-.
 "#,
             rendered
         )
@@ -736,7 +1006,7 @@ back two levels left without roffs
 
     #[test]
     fn it_roffs_examples() {
-        let roff = Roff::new("test-examples", 3).section(
+        let roff = Roff::new("test-examples", SectionNumber::LibraryCalls).section(
             "BASE SECTION",
             vec![
                 RoffNode::text("Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vivamus quis malesuada eros."),
@@ -752,9 +1022,7 @@ back two levels left without roffs
 
         let rendered = roff.to_string().unwrap();
         assert_eq!(
-            r#".TH "test\-examples" "3"
-.
-
+            r#".TH test\-examples 3
 .SH "BASE SECTION"
 Lorem ipsum dolor sit amet, consectetur adipiscing elit\. Vivamus quis malesuada eros\.
 .EX
@@ -764,7 +1032,8 @@ if x\.len() > 0 {
 	println!("{}", x);
 }
 
-.EE"#,
+.EE
+"#,
             rendered
         )
     }
@@ -780,4 +1049,97 @@ Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vivamus quis malesuada
 Lorem ipsum dolor sit amet, consectetur adipiscing elit\\. Vivamus quis malesuada eros\\.",
         )
     }
+
+    #[test]
+    fn synopsis_works() {
+        let roff = Roff::new("test-synopsis", SectionNumber::Miscellaneous).section(
+            "SYNOPSIS",
+            vec![
+                RoffNode::synopsis("ls", ["lists files in the given".roff(), "path".roff().italic(), ".".roff()],
+                vec![
+                    SynopsisOpt::new("-l").description(["use a long listing format"]),
+                    SynopsisOpt::new("-L, --dereference").description(["when showing file information for a symbolic link, show information for the file the link references rather than for the link itself"]),
+                    SynopsisOpt::new("--block-size").argument("SIZE").description(["with -l, scale sizes by SIZE when printing them"]),
+                ]
+                )
+            ],
+        );
+
+        let rendered = roff.to_string().unwrap();
+        assert_eq!(
+            r#".TH test\-synopsis 7
+.SH SYNOPSIS
+.SY ls
+lists files in the given\fIpath\fR\.
+
+.OP \-l
+use a long listing format
+
+.OP "\-L, \-\-dereference"
+when showing file information for a symbolic link, show information for the file the link references rather than for the link itself
+
+.OP \-\-block\-size SIZE
+with \-l, scale sizes by SIZE when printing them
+.YS
+"#,
+            rendered
+        )
+    }
+
+    #[test]
+    fn urls_and_emails_work() {
+        let roff = Roff::new("test-urls", SectionNumber::Miscellaneous).section(
+            "URLS",
+            vec![
+                RoffNode::url("GitHub", "https://github.com/vv9k/roffman"),
+                RoffNode::url("crates.io", "https://crates.io/crates/roffman"),
+                RoffNode::url("docs.rs", "https://docs.rs/roffman"),
+                RoffNode::email("John Test", "test@invalid.domain"),
+            ],
+        );
+
+        let rendered = roff.to_string().unwrap();
+        assert_eq!(
+            r#".TH test\-urls 7
+.SH URLS
+.UR https://github\.com/vv9k/roffman
+GitHub
+.UE
+.UR https://crates\.io/crates/roffman
+crates\.io
+.UE
+.UR https://docs\.rs/roffman
+docs\.rs
+.UE
+.MT test@invalid\.domain
+John Test
+.ME
+"#,
+            rendered
+        )
+    }
+
+    #[test]
+    fn special_strings_work() {
+        let roff = Roff::new("test-strings", SectionNumber::Miscellaneous).section(
+            "STRINGS",
+            vec![
+                RoffNode::left_quote(),
+                RoffNode::text("this is some example quoted text."),
+                RoffNode::right_quote(),
+                RoffNode::text(" "),
+                RoffNode::registered_sign(),
+                RoffNode::text(" roffman"),
+                RoffNode::trademark_sign(),
+            ],
+        );
+
+        let rendered = roff.to_string().unwrap();
+        assert_eq!(
+            r#".TH test\-strings 7
+.SH STRINGS
+\*(lqthis is some example quoted text\.\*(rq \*R roffman\*(Tm"#,
+            rendered
+        )
+    }
 }
